Documentation
¶
Overview ¶
Package websocketencoder exists to solve a problem with the default encoding in the socket.io library we're using. A PR is open [here](https://github.com/zishang520/socket.io-go-parser/pull/1) to build this functionality into the library, but in the meantime we've had to fork it.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func DeconstructPacket ¶
func DeconstructPacket(packet *parser.Packet) (pack *parser.Packet, buffers []types.BufferInterface)
DeconstructPacket Replaces every io.Reader | []byte in packet with a numbered placeholder.
Types ¶
type Placeholder ¶
type Placeholder struct {
Placeholder bool `json:"_placeholder" mapstructure:"_placeholder" msgpack:"_placeholder"`
Num int `json:"num" mapstructure:"num" msgpack:"num"`
}
Placeholder
Click to show internal directories.
Click to hide internal directories.